REBOL [
	Title: "REBOL/Services - Client API Functions"
	Author: "Carl Sassenrath"
	Notice: "Confidential source code. Under development."
	Copyright: "REBOL Technology 2005-2008, All rights reserved."
]

;-- Client API Functions -----------------------------------------------------

open-service: func [
	"Open a client service port. Returns port value."
	service [url! block!] "Service to connect to"
	/options opts "Configuration options (as object block)"
	/local spec config handler
][
	spec: make port! service
	config: make client-config any [opts []]
	config/flags: copy config/flags
	either handler: select client-handlers spec/scheme [
		config/handler-context: handler ; provides a path to sender function
		spec: handler/opener spec config
	][
		do-error ["Unsupported port type (scheme):" spec/scheme]
	]
	spec ; (note: may not match original scheme)
]

close-service: func [
	"Close an open service port."
	port [port! none!]
][
	; Should we abort all pending requests?
	; Do we need a /purge option?
	if not port? port [exit]
	attempt [close port]
	port/status: none
	debug tcp ['close-service port/host port/port-id]
]

send-service: func [
	"Send request to a REBOL service. Return req-message object."
	service [url! block! port!] "Service to connect to"
	request [block! none!] "Command request"
	/action callback [function! block!] "async callback func [service result req]"
	/local session
][
	request: compose/deep request

	; If simple command, wrap a block around it:
	request: either block? request/1 [copy request][reduce [request]]

	; If service is not open, open it now:
	if not port? service [
		service: open-service service ; block or url
;		session: session-service? service
	]

	; Queue the request and send it if necessary:
	queue-request service request :callback
	; RETURNS req object
]

on-service: func [
	"Specify a callback function for a service port."
	port [port!]
	callback [function!] "Callback: func [port req reply]"
][
	port/user-data/callback: :callback
]

query-service: func [
	"Return result from prior send-service or NONE."
	req [object!] "Request object returned from send-service"
][
	if req/state = 'result [req/result] ; else return NONE
]

wait-service: func [
	"Wait for result from prior send-service and return it."
	req [object!] "Request object returned from send-service"
	/timeout count [number!]
	/local port wait-block
][
	;req/wakeup: true ???
	if not port: req/port [return none]
	port/user-data/config/handler-context/set-wakeup port true
	forever [
		if req/state = 'result [
			;port/wakeup?: none
			port/user-data/config/handler-context/clear-wakeup port
			if req/result/1 <> 'done [
				throw-service reform [
					"command failed:"
					any [
						select req/result 'fail
						select req/result 'error
						"unknown"
					]
				]
			]
			return req/result
		]
		if req/state = 'aborted [
			;port/wakeup?: none
			port/user-data/config/handler-context/clear-wakeup port
			throw-service "aborted request"
		]
		; State must be 'queued or 'sent, so wait:
		either count [
			if not port? wait [req/port count] [
				;port/wakeup?: none
				port/user-data/config/handler-context/clear-wakeup port
				return none
			]
		][
			wait [req/port any [req/port/timeout 60]] ; port/wakeup? flag causes a return
		]
	]
]

abort-service: func [
	"Abort a prior send-service request. Returns TRUE if successful."
	req [object!] "Request object returned from send-service"
][
	if req/state = 'queued [
		remove find req/port/user-data/send-queue req
		req/state: 'aborted
		return true
	]
	false
]

do-service: func [
	"Send request to a REBOL service and wait for response. Return result."
	service [url! block! port!] "Service to connect to"
	request [block! word! none!] "Command request"
	/local req result
][
	req: send-service service request
;ppp: req/port ???
	result: wait-service req
	if not port? service [close-service req/port]
	result
]

login-service: func [
	"Authenticate user/pass and begin encrypted session. (blocks)"
	service [url! block! port!] "Service to connect to"
	user "User identifier"
	pass "Password string or hash"
	/local req data result
][
	req: send-service service [login (user)]
	result: wait-service req
	if all [
		result
		parse req/result ['done skip 'ok set data block!]
		data/1 = 'login
	][
		; data: [challenge salt encoding crypt-len]
		session: session-service? service
		remove data ; remove 'login word
		session/challenge: data
		if string? pass [pass: make-pass pass data/2]
		session/crypt-len: data/4
		crypt-session session data/3 pass
		session/login-date: now
		req/port
	]
	; Return service port or NONE.
]

logout-service: func [
	"Conclude authenticated service session."
	port [port! none!]
	/local req
][
	if not port? port [return false]
	req: send-service port [logout]
	wait-service req
	reset-session port/user-data
	req/result/1 = 'done
]

throw-service: func [
	"temporary"
	msg
][
	do make error! msg
]
