REBOL [
	Title: "REBOL/Services - Encryption"
	Author: "Carl Sassenrath"
	Notice: "Confidential source code. Under development."
	Copyright: "REBOL Technology 2005-2008, All rights reserved."
]

;-- Packet Encryption --------------------------------------------------------
; 
;   Packet encryption types:
;
;       E0 - no encryption (not implemented)
;       E1 - Cloak - REBOL public key
;       E2 - Cloak - shared secret key
;       E3 - Cloak - unique session key
;       E4 - AES   - shared secret key
;       E5 - AES   - unique session key
;       E6 - RSA   - private/public key pair
;
; ---General encryption method:
; 
;   1. client + key (login) ----> server + key
;   2. client + key <---- server + key (challenge)
;   3. client + key + auth (cmd) ----> server + key + auth
; 
; The client sends logon packet using a shared secret key (SSK).
; The server responds with a challenge packet using SSK.
; Both sides generate a session key based on SSK and auth keys.
; 
; ---Method 1: Cloak based:
; 
; The SSK is a built-in part of distribution. The encryption method is
; CLOAK with weak key. After authentication, CLOAK is used with medium
; key based on authentication keys.
; 
; ---Method 2: RSA & AES based:
; 
; The SSK is the RSA public key on client and thr RSA private key on server.
; This allows certification of the server.
; 
; The encryption method is RSA for logon. After authentication the session
; method is AES with a strong key based on RSA + authentication keys.
; 
; ---Other Notes:
; 
; 1. The packet header indicates encryption is being used and at what level.
; 
; 2. Both methods can allow INFO service requests without encryption.
; That is a server configuration option.
; 
; 3. Neither user password nor session key are ever sent over the net.
; 
; 4. The challenge is SHA1 crypto-hash-based and is always different,
; preventing the recording & playback of the logon packet sequence.
; 
; 5. After logon the request header (encrypted) includes session request
; sequence number to prevent out-of-sequence packet playback during same
; session key.
;
; 6. RSA and AES features only available in REBOL/Pro or /Command. See
; http://www.REBOL.com/products.html. Export restrictions may apply.

; Default-Key: This key is only used for OPEN and PRELOG command
; services. DO NOT change this key if you want to allow any OPEN
; requests such as INFO commands for publishing your service. You can
; still operate a private service but allow open INFO requests. To
; do so, change the CRYPT-KEY in your configuration.

default-key: checksum/secure "REBOL/Services default encryption key"

aes-method: 'rijndael

;-- Packet encoding levels/flags:

pe-open:       1 ; open access
pe-priv-cloak: 2 ; use private cloak key
pe-sess-cloak: 3 ; use session cloak key
pe-priv-aes:   4 ; use private aes key
pe-sess-aes:   5 ; use session aes key
pe-priv-rsa:   6 ; use rsa keys

pe-mask:      31 ; mask for above

pe-base16:    32 ; flag
pe-base64:    64 ; flag


encode-packet: func [
	"Apply encryption and byte encoding."
	session [object!]
	data
][
	; For encode, session id comes from session object.
	;print "ENCODE-PACKET"
	;print [session/id session/seq session/remote-ip session/encode if integer? session/encode [key-type? session/encode]]
	switch key-type? session/encode [
		cloak [
			encloak/with as-binary data session/crypt-key
		]
		aes [
			insert session/encrypt-port data
			update session/encrypt-port
			copy session/encrypt-port
		]
		rsa [
			either session/rsa-key/d [ ; (if private key, then to client)
				rsa-encrypt/private session/rsa-key data ; server
			][
				rsa-encrypt session/rsa-key data ; client (public key)
			]
		]
	]
]

decode-packet: func [
	"Apply decryption and byte decoding."
	session [object!]
	data
][
	; For decode, session id comes from packet.
	;print "DECODE-PACKET"
	;print [session/id session/seq session/remote-ip session/encode if integer? session/encode [key-type? session/encode]]
	;print [session/header ph-encode? session/header key-type? ph-encode? session/header]
	switch key-type? ph-encode? session/header [
		cloak [
			decloak/with data session/crypt-key
		]
		aes [
			insert session/decrypt-port data
			update session/decrypt-port
			copy session/decrypt-port
		]
		rsa [
			either find session/rsa-key/d [ ; (if public key, then from client)
				rsa-encrypt/private/decrypt session/rsa-key data ; server
			][
				rsa-encrypt/decrypt session/rsa-key data ; client (public key)
			]
		]
	]
]

key-type?: func [
	"Returns CLOAK, AES, or RSA depending on encoding integer."
	encoding [integer!]
][
	any [
		pick [
			cloak ; public
			cloak ; private
			cloak ; session
			aes   ; private
			aes   ; session
			rsa   ; logon
		] encoding and pe-mask
		'cloak
	]
]

open-crypt-ports: func [
	"Opens two AES (Rijndael) encryption ports."
	session
	/local bits
][
	close-crypt-ports session
	bits: session/crypt-len
	session/encrypt-port: make port! [
		scheme: 'crypt
		direction: 'encrypt
		key: session/crypt-key
		algorithm: aes-method
		strength: bits
		padding: true
	]
	open session/encrypt-port
	session/decrypt-port: make port! [
		scheme: 'crypt
		direction: 'decrypt
		key: session/crypt-key
		algorithm: aes-method
		strength: bits
		padding: true
	]
	open session/decrypt-port
]

close-crypt-ports: func [
	session
][
	if not session [exit]
	if session/encrypt-port [
		attempt [close session/encrypt-port]
		session/encrypt-port: none
	]
	if session/decrypt-port [
		attempt [close session/decrypt-port]
		session/decrypt-port: none
	]
]

make-rsa-key: func [
	"Returns RSA object from public key cypher (prime base 3)."
	public-key [binary!] "Public part of the RSA key."
	/local key
][
	key: rsa-make-key
	key/e: 3
	key/n: public-key
	key
]

make-session-key: func [
	"Generates session key on successful login (both sides)."
	session [object!]
	auth "Pass hash"
	/local key user
][
	if not block? session/challenge [exit]
	; Generate session key based on user's password hash + server key + challenge:
	key: join auth any [session/config/priv-key session/config/open-key]
	if session/rsa-key [
		append key either object? rsa-key [
			session/rsa-key/n
		][
			session/rsa-key
		]
	]
	append key checksum/secure session/challenge/1
	session/crypt-key: checksum/secure key
]


