REBOL [
	Title: "REBOL/Services - Server Configuration"
	Author: "Carl Sassenrath"
	Notice: "Confidential source code. Under development."
	Copyright: "REBOL Technology 2005-2008, All rights reserved."
]

;-- Service Configuration Objects --------------------------------------------
;
;   Defines the general parameters of the server and its services.
;   For a full description of all values, view the rebser.cfg file.

config-file: %rebser.cfg  ; default configuration file

;-- Service Information Object -----------------------------------------------
;
;   This object provides public information related to the service.
;   The INFO command can be used to query this information for the
;   purposes of listing services in a directory, etc.

service-info: context [
	title: "Generic REBOL Service"
	organization: "REBOL Free Site"
	summary: "This is a default REBOL service."
	notes: "Generic."
	type: 'tcp			; tcp | cgi
	version: 1.0.0
	url: none
	port: 2997
	language: [EN]		; A block of language words
	contact: none		; RSP contact info, email, address, phone.
	copyright: none
	license: none
	private: none		; true or false
	benchmark: none
	bandwidth: none
	system: none
	clients: none
	created: none
	updated: none
	client-version: 0.0.0
]

;-- Service Configuration Object ---------------------------------------------
;
;   This object provides the private configuration information for a
;   service. It is accessed at high frequency and is kept minimal.

server-config: context [
	flags: [server]
	service-info: none		; Public info about this service server
	services: []			; List of service names
	service-dir: %services/	; Where to find services
	service-list: []		; List of service objects (initialized)
	port-timeout: 20
	max-header:  2000		; Maximum header. Allows for trash at start.
	max-request: 100'000	; Maximum request size. Abort if greater.
	max-payload: 8'000'000	; Maximum payload size. Abort if greater.
	session-limit: 100
	session-timeout: 0:2:00
	session-list: []		; active sessions [id session-obj ...]
	session-id: 0			; next session id to use
	session-dir: %sessions/	; used by transient servers like CGI
	ip-allow: none			; [1.2.3.4 5.6.0.0 mask 255.255.0.0]
	ip-deny: none			; ALL | [1.2.3.4 5.6.0.0 mask 255.255.0.0]
	encode: pe-open			; packet encoding method (see pe values)
	open-key: default-key	; encryption key used for open access (CLOAK)
	priv-key: none			; encryption key used for private access (CLOAK, AES)
	rsa-key: none			; optional RSA key for certification (binary | object)
	base-dir: none			; base for service operations (default is start dir)
	limits: none
	clients: none
	debug-watch: true
	log-watch: true
	log-file: %log.txt
	user-file: %userbase.rdb
]

load-config: func [
	"Load an external server configuration. Return Config."
	config [block! file! none!] "File, object block, or services list"
	transient "Set TRUE when transient (e.g. CGI)"
	/local conf files
][
	base-dir: what-dir  ; Store base directory for use by admin service (updates)

	switch type?/word config [
		file! [
			if not exists? config [
				throw-service "Error: Missing configuration file"
			]
			config: load config
		]
		block! [
			; If block has no set-words, it is a list of services:
			if not find config set-word! [
				config: compose/only [services: (config)]
			]
		]
		none! [config: []]
	]
	conf: construct/with config server-config
	conf/flags: copy conf/flags

	; Use the current dir for services if none is specified:
	if none? conf/base-dir [conf/base-dir: what-dir]

	; Disable debug to prevent output (e.g. when in CGI):
	if transient [
		append conf/flags 'transient
		conf/debug-watch: false
	]

	; Start logging, according to the config settings:
	init-log conf
	debug config ['load-config config]

	conf/service-info: service-info
	conf/session-dir: dirize conf/session-dir

	; If no encoding is configured, use open encoding:
	if not conf/encode [
		conf/encode: pe-open
		conf/open-key: checksum/secure default-key
	]

	; Session tracking:
	conf/session-list: copy []
	conf/session-id: 0
	if transient [
		make-dir conf/session-dir ; ignored if already exists
		sweep-sessions conf
		files: attempt [sort load conf/session-dir]
		if all [files not empty? files] [
			; !!! PROBLEM: how to wrap session id on integer overflow?
			conf/session-id: to-integer last files
		]
	]
	conf
]

;-- Server Scheme Handler List
;
;   For the protocol scheme that you want to serve, there must be a handler
;   object in this list. Examples: TCP, CGI

server-handlers: []

add-server-handler: func [
	spec [block!]
][
	spec: context spec
	repend server-handlers [spec/scheme spec] ; (error ok if no scheme field)
]
